;
; close.asm - CLOSE kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Close (A=logical file number)
;
; Used registers: A X (Y preserved only if closeing recently opened file. This is used in some buggy software...)
; Status: C=0
; Status byte response:
; ST=$03 timeout
; ST=$80 if device not present
; Notes:
; Secondary address 128..255 then no communication to the serial bus
;**************************************************
newclose	.proc
		tax
		jsr search_logical_X_close
		bne close_ano3          ;File already closed

		jsr search_buffer_A_readfiletables
		beq close_ano2		;Ok found buffer for file

		#ldx_l_a open_files_currentp_exp
		lda open_files_info_exp,x
		cmp #INFO_KERNAL
                php
                jsr close_real
                plp
		bne close_ano3		;non-kernal file

		lda ActualDev
                jsr chkinout_driveconv

		txa
		beq close_ano3 		;keyboard
		cmp #3
		beq close_ano3 		;screen
		bcc tape_rs232   	;tape, etc
		lda ActualSecAddr
		bmi close_ano3		;invalid

		#psr $f648
tback		txa

close_ano3	clc
		jmp exit2basic		;Do the kernal stuff...

tape_rs232 	#lda0
		pha
                #adc1
		sta OpenFiles		;Fake kernal about files table
		#psr $f2a7
                gcc tback               ;Do the kernal stuff...

close_real	lda ActualLogical
		#ldx_l_ buffer_input_exp
		jsr closekill_inout	;Disconnect I/O from closed file
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jsr chkin_doch
		.else
		#stx_l_ buffer_input_exp
		.fi
		#ldx_l_ buffer_output_exp
		jsr closekill_inout
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jsr chkout_doch
		.else
		#stx_l_ buffer_output_exp
		.fi

		#ldx_l_a open_files_currentp_exp
		#lda0
		sta open_files_logical_exp,x;Now file is closed
		rts

closekill_inout	cmp buffer_logical_exp,x
		bne close_an1
		ldx #IGNOREINOUT
close_an1	rts

close_ano2	#phy_a
		jsr OP_save_zeropage_scpu01up
		jsr XIT_close
		jsr OP_restore_zeropage_scpu01down
		#ply_a

		jsr close_real
                .if SUPERCPU=1
                lda buffer_currentp_exp
                tax
                tdc
                .else
		ldx buffer_currentp_exp
                .fi
		sta buffer_logical_exp,x;Now file buffer is freed

		#jmps close_ano3

XIT_close	#xit_init

		#ldx_l_a buffer_currentp_exp
                jsr fileopprepare.in
		lda buffer_info_exp,x
		cmp #INFO_STATUS
		beq XIT_commandchnl.in
                #andlsr INFO_MSK
		tay
		lda buffer_type_exp,x
		cmp #BUFFERMODE_FILE
		beq XIT_do_file
                cmp #BUFFERMODE_PCLINK
                beq pclink_close2
		rts

pclink_close2	
;		.if (PCLINK_TYPE=3)
;                php				;not needed, one byte received only
;                sei
;                .fi
		jsr flush_pclink
		sec
;                .if (PCLINK_TYPE=3)
;	         jsr pclink_openclose
;                plp				;not needed, one byte received only
;                rts
;                .else
		jmp pclink_openclose
;                .fi

XIT_do_file	lda #INFO_IGNORE	;be safe
		sta buffer_info_exp,x

		lda sowhat,y
                beq nothing
                php
                jsr update_filelen
                plp
                bpl noflush

		jsr XIT_flush_buf

noflush 	jsr restore_file_state

		jsr XIT_flush_free_and_tree

		#cnext cfs_file_dirsect_exp , next_sector

		lda #ram_exp_dir
		jsr XIT_read_a_sector	;-=> error

                lda cfs_file_diroffs_exp
                sta resolv_entry_offs_exp

		#ldx_l_a buffer_currentp_exp
		lda buffer_filesize1_exp,x
		sta szam+3
		lda buffer_filesize2_exp,x
		sta szam+2
		lda buffer_filesize3_exp,x
		sta szam+1
		lda buffer_filesize4_exp,x
		sta szam

                lda #FATTR_CLOSED
                clc
                jsr XIT_finalize_direntry;update timestamp and filesize
nothing         jmp LWOP_reporterror_ok
                
sowhat		.byte $80		;INFO_BLK_WRITE_DIRTY
		.byte $80		;INFO_BLK_RW_DIRTY
		.byte $80		;INFO_BLK_RW_EOF_DIRTY
		.byte $01		;INFO_BLK_WRITE
		.byte $01		;INFO_BLK_RW
		.byte $01		;INFO_BLK_RW_EOF
		.byte $00		;INFO_BLK_RW_EOF_FULL
		.byte $00		;INFO_BLK_WRITE_FULL
		.byte $00		;INFO_BLK_READ
		.byte $00		;INFO_BLK_R_EOF
		.byte $00		;INFO_RELATIVE
		.byte $00		;INFO_STATUS
		.byte $00		;INFO_KERNAL
		.byte $00		;INFO_IGNORE
		.pend
;---------------------------
;Create timestamp to [pom1]
;
OP_timestamp	.proc
		jsr LWOP_read_clock
                ldy #$1c		;timestamp offset
                lda sec_exp
		jsr bcd2bin
                #sta_zp_y pom1
                lda min_exp
		jsr iny_bcd2bin
                #sta_zp_y pom1
                lda year_exp
		jsr iny_bcd2bin
                clc
                adc #20
                cmp #100
                bcc *+4
                sbc #100
                and #%00111111
                #sta_zp_y pom1
                lda date_exp
		jsr iny_bcd2bin
                #sta_zp_y pom1
		
                lda hour_exp
		jsr bcd2bin
                jsr cc
                dey

                lda month_exp
		jsr bcd2bin

		asl
cc              asl
                asl
                asl
                pha
                asl
                asl
                #ora_zp_y pom1
                #sta_zp_y pom1
                dey
                pla
                and #%11000000
                #ora_zp_y pom1
                #sta_zp_y pom1
                rts
                .pend

flush_pclink	.proc
		lda buffer_info_exp,x
		lsr
                bcs none
		lda buffer_address_hi_exp,x
                sec
                sbc buffer_start,x
                bcc none
                tay
                lda buffer_address_low_exp,x
                tax
                inx
                bne *+3
                iny
;		sec
                jsr pclink_chrinout
none        	rts
		.pend
